#!/usr/bin/env python3
"""
geoms_tctools.py

"""

import os
from pathlib import Path
import numpy as np
import pycr8ascii



def check_for_version_name(g_obj):
    """
    Determine if DATA_SOURCE has a VERSION_NAME field

    """

    try:
        dsi = g_obj.ga_d['DATA_SOURCE']
        dsx = dsi.split('_')
    except KeyError:
        return ''

    if len(dsx) == 3:
        return dsx[2]

    return ''


def read_geoms_template(templatefile):
    """
    Read template file

    """

    with open(templatefile, 'r') as f:
        chGEOMSTemp = [s.rstrip("\n\r") for s in f.readlines()]

    return chGEOMSTemp


def check_data_source(g_obj):
    """
    Check data source

    """

    try:
        dsi = g_obj.ga_d['DATA_SOURCE']
        chHDFDS = dsi.split(';')
    except KeyError as ke:
        pass

    # Extract name and gas of data source
    chHDFDSDum1 = chHDFDS[0].split('_')
    chHDFDSDum = chHDFDSDum1

    # check if data source contains '.', then split
    chHDFDSDum2 = chHDFDSDum1[0].split('.')
    chHDFDSDum = chHDFDSDum2

    iSize1 = len(chHDFDSDum)

    chHDFName = chHDFDSDum[0]
    chHDFGas = chHDFDSDum[iSize1 - 1]

    return chHDFGas

def checkgas(chVarCheck, chSearch, chHDFGas):
    """
    Check if [GAS] is reported

    """

    return chVarCheck.replace(chSearch, chHDFGas)



def analyze_geoms_va(g_obj, chGEOMSTempVA, logfile):
    """
    Analyze variable attributes

    """

    tccode = ['0 (Passed template checks)', '1 (Failed template checks)']

    # get [GAS] form data source
    chHDFGas = check_data_source(g_obj)

    # size of GEOMS-TE VA
    iSize1 = len(chGEOMSTempVA[0].split(','))

    # size of GEOMS-TE VA
    iSize2 = len(chGEOMSTempVA)

    # size of HDF variables
    iSize3 = len(g_obj.sdss_d.keys())

    # size of HDF VA
    # Get number of attributes for 1st variable listed
    iSize4 = len(g_obj.sdss_d[g_obj.ga_d['DATA_VARIABLES'].split(';')[0]].va_l)

    # size of predefined VA
    iSize5 = iSize1 - 3           # size(chVA, /N_ELEMENTS )

    # decide on type of data template
    if iSize1 == 5:
        chVA = [ 'VAR_NAME', 'VAR_UNITS' ]
    if iSize1 == 6:
        chVA = [ 'VAR_NAME', 'VAR_UNITS', 'VAR_DATA_TYPE' ]
    if iSize1 == 7:
        chVA = [ 'VAR_NAME', 'VAR_UNITS', 'VAR_DATA_TYPE', 'VAR_DEPEND' ]


    # 4 elements read from HDF file, max 5 elements read from template
    chHDFVARed = np.zeros((iSize3,4), dtype=object)
    chGEOMSTempVARed = np.zeros((iSize2,5), dtype=object)
    chGEOMSTempVARedReplace = np.zeros((iSize2,5), dtype=object)

    chHDFVARed[:] = ''
    chGEOMSTempVARed[:] = ''
    chGEOMSTempVARedReplace[:] = ''

    goodexit = 0
    # Loop over data variables. Follow order from DATA_VARIABLES global attribute
    for iR3, var_key in enumerate(g_obj.ga_d['DATA_VARIABLES'].split(';')):
        for iR5 in range(iSize5):
            #iC4 = 0
            goodexit = 0
            for iC4 in range(iSize4):
                if g_obj.sdss_d[var_key].va_l[iC4] == chVA[iR5]:
                    goodexit = 1
                    break
                iC4 += 1

            if goodexit == 1:
                chHDFVARed[iR3,iR5] = g_obj.sdss_d[var_key].va_v[iC4]
            else:
                if iR5 == 0:
                    chHDFVARed[iR3,iR5]= 'Invalid/Unknown'


    # Read GEOMS-TE VA and assign values
    # The reading of the tempate CSV files strongly depends on its format.
    for iR2 in range(iSize2):
        chGEOMSTempVATmp = chGEOMSTempVA[iR2].split(',')

        # template variable name
        chGEOMSTempVARed[iR2,0] = chGEOMSTempVATmp[1]

        # LIDAR 2011-04-23
        if iSize1 >= 5:
            # template variable units
            chGEOMSTempVARed[iR2,1] = chGEOMSTempVATmp[2]
            # template variable requirement
            chGEOMSTempVARed[iR2,3] = chGEOMSTempVATmp[len(chGEOMSTempVATmp) - 1]

        # MWR, FTIR 2011-04-23
        if iSize1 >= 6:
            # template variable units
            chGEOMSTempVARed[iR2,1] = chGEOMSTempVATmp[2]
            # template variable data type
            chGEOMSTempVARed[iR2,2] = chGEOMSTempVATmp[3]
            # template variable requirement
            chGEOMSTempVARed[iR2,3] = chGEOMSTempVATmp[len(chGEOMSTempVATmp) - 1]

        # FTIR 2011-05-18
        if iSize1 >= 7:
            # template variable units
            chGEOMSTempVARed[iR2,1] = chGEOMSTempVATmp[3]
            # template variable data type
            chGEOMSTempVARed[iR2,2] = chGEOMSTempVATmp[4]
            # template variable requirement
            chGEOMSTempVARed[iR2,3] = chGEOMSTempVATmp[len(chGEOMSTempVATmp) - 1]
            # template variable dependencies [dimensions]
            chGEOMSTempVARed[iR2,4] = chGEOMSTempVATmp[2]

    # Check if search string found in template and count optional attributes

    # TODO: to be implemented
    # var1

    #   chVarOpt1 = ''
    #   chSearch1 = '[SOLAR|LUNAR]'
    #   iPos1 = check_opt_var(chGEOMSTempVARed(*,0), chSearch1)

    # if (iPos1 ge 0) then begin
    #    chVarOpt1 = get_optional_vars(chHDFVARed(*,0), chSearch1)
    #    printf, lu, '  INFORMATION: Variable option '+chSearch1+' found and analyzed.'
    # end

    #                               ; var2
    # chVarOpt2 = ''
    # chSearch2 = '[PRESSURE|TEMPERATURE]_INDEPENDENT_INITIALIZATION'
    # iPos2 = check_opt_var(chGEOMSTempVARed(*,0), chSearch2)

    #                                   go through all GEOMS-TE and replace [GAS] and optional variables e.g. [SOLAR|LUNAR]; create new array
    for iR2 in range(iSize2):
        # Replacement checks
        # 1) check if [GAS] is reported, that data source [GAS] matches gases in VA
        chVarCheck = checkgas( chGEOMSTempVARed[iR2,0], '[GAS]', chHDFGas )
        # 2) replace/substitute optional attributes
        # STRREPLACE, chVarCheck, chSearch1, chVarOpt1
        # STRREPLACE, chVarCheck, chSearch2, chVarOpt2

        chGEOMSTempVARedReplace[iR2,0] = chVarCheck
        chGEOMSTempVARedReplace[iR2,1] = chGEOMSTempVARed[iR2,1]
        chGEOMSTempVARedReplace[iR2,2] = chGEOMSTempVARed[iR2,2]
        chGEOMSTempVARedReplace[iR2,3] = chGEOMSTempVARed[iR2,3]
        chGEOMSTempVARedReplace[iR2,4] = chGEOMSTempVARed[iR2,4]



    # Check diffs GEOMS-TE -> HDF
    # Do limited check: check if variable is present; all other checks done by HDF -> GEOMS-TE
    iCheckTotal = 1
    for iR2 in range(iSize2):
        iCheck = 0
        for iR3  in range(iSize3):
        # check for plain existence of data variable
            if chGEOMSTempVARedReplace[iR2,0] == chHDFVARed[iR3, 0]:
                iCheck = 1
                iVarNr = iR3

        # Check if mandatory
        if iCheck == 0:
            if chGEOMSTempVARed[iR2, 3] == 'o':
                with open(logfile, 'a') as f:
                    f.write('  INFORMATION: '+chGEOMSTempVARedReplace[iR2,0]+' optional data variable is not found in HDF file.\n')
                iCheck = 1

        # check if conditional
        if chGEOMSTempVARed[iR2,3][:4] == 'x if':
            chCondition = chGEOMSTempVARed[iR2,3].split(' ')

        # TODO: to be implemented

         #  replacement checks
        #   1) check if [GAS] is reported, that data source [GAS] matches gases in VA
#        chConditionCheck = checkgas( chCondition(2), '[GAS]', chHDFGas )
# #			; 2) replace/substitute optional attributes
#         STRREPLACE, chConditionCheck, chSearch1, chVarOpt1
#         STRREPLACE, chConditionCheck, chSearch2, chVarOpt2

# # check if variable condition found in variable list
#         iPos = where(chConditionCheck eq chHDFVARed(*,0))

# # DEBUG
# #print, chCondition(2)+': '+chConditionCheck+': '+chHDFVARed(*,0)

#         if ( ( chCondition(2) eq chHDFGas ) or ( iPos ge 0 ) ) then begin
#            if (iCheck eq 0) then begin
# #					print, '  ERROR: '+chGEOMSTempVARed(iR2,0)+' conditional data variable is not found in HDF file.'
#               chError = '  ERROR: '+chGEOMSTempVARedReplace(iR2,0)+' conditional data variable is not found in HDF file, and required by template.'
#               print, chError
#               printf, lu, chError
#               iCheck = 2        ;1 (changed v1.3)
#            end
#         end else if (iPos eq -1) and (iCheck eq 1) then begin
# #printf, lu, '  ERROR: '+chGEOMSTempVARedReplace(iR2,0)+' conditional data variable is found in HDF file, but does not satisfy the condition in the template.'
# #iCheck = 2 ;1 (changed v1.3)
#            printf, lu, '  INFORMATION: '+chGEOMSTempVARedReplace(iR2,0)+' conditional data variable is found in HDF file, and treated as an optional variable.'
#            iCheck = 1 ;(changed v1.34 due to GEOMS ruling allowing conditional variables to be treated as optional when they do not meet the condition)
#         end else begin
#            printf, lu, '  INFORMATION: '+chGEOMSTempVARedReplace(iR2,0)+' conditional data variable is not found in HDF file, and not required by template.'
#            iCheck = 1
#         end
#      end

        # check if variable exists
        if iCheck == 0:
            chError = '  ERROR: '+chGEOMSTempVARedReplace[iR2,0] + ' data variable is not found in HDF file.'
            print(chError)
            with open(logfile, 'a') as f:
                f.write(chError + '\n')
        else:
            if iCheck == 2:
                iCheck = 0 # added v1.3

        iCheckTotal *= iCheck


    # check diffs HDF -> GEOMS-TE

    for iR3 in range(iSize3):
        iCheck = 0

        for iR2 in range(iSize2):
            # replacement checks
            # 1) check if [GAS] is reported, that data source [GAS] matches gases in VA
            chVarCheck = checkgas(chGEOMSTempVARed[iR2,0], '[GAS]', chHDFGas)
            # 2) replace/substitute optional attributes
            # STRREPLACE, chVarCheck, chSearch1, chVarOpt1
            # STRREPLACE, chVarCheck, chSearch2, chVarOpt2

            # check for plain existence of data variable
            if (chVarCheck == chHDFVARed[iR3, 0]):
                iCheck = 1
                iStor2 = iR2


        if iCheck == 1:
            #  check validity of units, data type and dependencies
            # TODO: to be implemented
            pass
        else:
            chError = '  ERROR: ' + str(chHDFVARed[iR3,0]) + ' data variable is not found in template.'
            print(chError)
            with open(logfile, 'a') as f:
                f.write(chError + '\n')

        iCheckTotal *= iCheck




    if iCheckTotal == 1:
        iCheckOut = 0
    else:
        iCheckOut = 1


    with open(logfile, 'a') as f:
        f.write('\n' + tccode[iCheckOut] + '\n\n')

    return iCheckOut

def geoms_tctools(templatefile, hdffile, QA_error_code):
    """
    geoms_tctools

    """

    hdf_dir =  os.path.dirname(os.path.abspath(hdffile))
    geoms_obj = pycr8ascii.Geoms(hdffile, hdf_dir)


    tmp_log_file = os.path.join(hdf_dir, 'pycr8ascii.log')
    if os.path.isfile(tmp_log_file):
        os.remove(tmp_log_file)

    geoms_obj.set_global_log_file_name(tmp_log_file)
    geoms_obj.get_data()

    chVN_value = check_for_version_name(geoms_obj) # check for existence of version name in DATA_SOURCE

    # find log file (see geoms_tools.pro)
    logfile = Path(os.path.abspath(hdffile)).with_suffix('.log')

    chGEOMSTemp = read_geoms_template(templatefile)
    chMessage = 'Analyzing compliance to GEOMS template: ' + os.path.basename(templatefile)
    print(chMessage)
    with open(logfile, 'a') as f:
        f.write(chMessage + '\n')

    TC_error_code = analyze_geoms_va(geoms_obj, chGEOMSTemp, logfile)

    if chVN_value == '':
        # print out QA/TC result because no need to do version name check
        chMessage = str(QA_error_code + 3*TC_error_code + 4 ) + ' (Total QA/TC error)'
        with open(logfile, 'a') as f:
            f.write(chMessage + '\n')

    return TC_error_code
